/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "gctl/io.h"
#include "../lib/potential.h"

using namespace gctl;

int main(int argc, char const *argv[])
{
	array<vertex3dc> node;
	array<mag_tetrahedron_ren17> ele17;
	array<mag_tetrahedron> ele;
	read_Tetgen_node("data/prism/prism.1", node);
	read_Tetgen_element("data/prism/prism.1", ele17, node);
	read_Tetgen_element("data/prism/prism.1", ele, node);

	array<magtet_para_ren17> mag_para17;
	array<magtet_para> mag_para;
	array<double> sus(ele17.size(), 0.1);
	callink_magnetic_para_earth(ele17, mag_para17, 60.0, 30.0);
	callink_magnetic_para_earth(ele, mag_para, 60.0, 30.0);

	// 设置观测点位
	array<point3dc> obsp;
	gridspace(point3dc(-30, 0, 5), point3dc(30, 0, 5), point3dc(0, -30, 5), point3dc(0, 30, 5), 121, 121, obsp);

	// 正演计算
	array<double> obsval;
	array<point3dc> obsgrad, obsgrad2;
	array<tensor> obstensor, obstensor2;

	// 正演磁位数据
	magobser(obsval, ele17, obsp, sus, ShortMsg);
	// 保存网格
	gctl::save_netcdf_grid("out_magobser_tetra", obsval, 121, 121, -30.0, 0.5, -30.0, 0.5, "x", "y", "Potential_17");

	// 正演磁分量数据
	magobser(obsgrad, ele17, obsp, sus, ShortMsg);
	// 保存网格
	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].x;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "Bx_17");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].y;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "By_17");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].z;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "Bz_17");

	// 正演磁分量数据
	magobser(obsgrad2, ele, obsp, sus, ShortMsg);
	// 保存网格
	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad2[i].x;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "Bx");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad2[i].y;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "By");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad2[i].z;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "Bz");

	// 计算并保存差值
	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].x - obsgrad2[i].x;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "Diff_Bx");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].y - obsgrad2[i].y;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "Diff_By");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].z - obsgrad2[i].z;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "Diff_Bz");

	gctl::_1d_array deltaT;
	magnetic_components2deltaT(obsgrad2, deltaT, 60, 30);
	gctl::append_netcdf_grid("out_magobser_tetra", deltaT, "x", "y", "DeltaT");
	return 0;
}