/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "gctl/io.h"
#include "gctl/geometry.h"
#include "../lib/potential.h"

using namespace gctl;

int main(int argc, char const *argv[]) try
{
    std::string data_dir = "/Volumes/DataArchive/Earth/Global/MagneticModels/EMM2017_Linux";
    text_content tc;
    tc.head_num_ = 1;
    tc.load_text(data_dir + "/EMM2017", ".COF");

    int n, m;
    _1d_array P, C, S;
    std::stringstream tmp_ss;

    C.resize(tc.lines_.size());
    S.resize(tc.lines_.size());
    for (size_t i = 0; i < tc.lines_.size(); i++)
    {
        str2ss(tc.lines_[i], tmp_ss);
        tmp_ss >> n >> m >> C[i] >> S[i];
    }

    power_spectrum_shc(P, C, S, 6371200, 6371200, 1, 65);

    for (size_t i = 0; i < P.size(); i++)
    {
        std::cout << i + 1 << " " << P[i] << "\n";
    }
    
    return 0;
}
catch (std::exception &e)
{
    GCTL_ShowWhatError(e.what(), GCTL_ERROR_ERROR, 0, 0, 0);
}